#!/bin/env python
"""The dbcheck is a sqlquery - execution time based one.
Often it's possible to check the business of a db server better by give him
some sql - queries and compare the executiontime with a given value then only
looking for the load. This modul provide that.

Now only MySQL and PostgreSql - Server are supported.

Common informations:
    This software is made for usage with the nagios monitoring system.
    If you change something or add new features please send a message to
    the author.

Licence:
    This program is free software; you can redistribute it and#or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

Additional informations:
    This software supports two diffrent Postgres dbapis. Best is use python 2.2.x
    or 2.3 with the PyPgSQL 2.4 dbapi. As MySQL - interface use MySQLdb 0.9.2.
    The pgdb - interface altough works stable but i only found it on Mandrake Linux machines, so
    i normally use PyPgSQL.
    The standard installation of debians woody will not work. There are some weird problems out of
    the PyPgSQL modul. So better do it by hand.

Author: Andy-Kim Moeller <kim@ypsilon.net>
"""

import sys
import getopt
from NagiosUtils import NagiosUtils
from time import time

nagios = NagiosUtils()

usage = """check_sql_timer.py by kim@ypsilon.net

./check_sql_timer.py -H<SERVER> -u<USER> -p<PASSWD> -d<DATABASE> -w<SECONDSTOWARNING> -c<SECONDSTOCRITICAL> -t[MYSQL|PGSQL] [--help|-h] SQLFILES

Seconds can also be a float value (like 0.50)
"""

# GETTING THE COMMANDLINE OPTIONS
try:
	opts, args = getopt.getopt(sys.argv[1:], "h:H:u:p:d:w:c:t:", ["help","host=","user=","passwd=","database=","warning=","critical=","type="])
except:
	import traceback
	traceback.print_exc(file=sys.stdout)
	nagios.tell('UNKNOWN',usage)
# CHECKING IF FILES ARE GIVEN
if len(args) < 1:
	nagios.tell('UNKNOWN',usage)
# CHECKING IF ALL OPTIONS ARE SET
server = ''
user = ''
passwd = ''
database = ''
warning = ''
critical = ''
dbtype = ''
for opt,value in opts:
	if opt in ['-h','--help']:
		nagios.tell('UNKNOWN',usage)
	if opt in ['-H','--host']:
		server = value
	if opt in ['-u','--user']:
		user = value
	if opt in ['-p','--passwd']:
		passwd = value
	if opt in ['-d','--database']:
		database = value
	if opt in ['-w','--warning']:
		try:
			warning = float(value)
		except:
			pass
	if opt in ['-c','--critical']:
		try:
			critical = float(value)
		except:
			pass
	if opt in ['-t','--type']:
		if value.upper() in ['MYSQL','PGSQL']:
			dbtype = value.upper()
if (server == '') or (user == '')  or (database == '') or (warning == '') or (critical == '') or (dbtype == ''):
	nagios.tell('UNKNOWN',usage)

# DOING THE PGSQL OR MYSQL SPECIFIC STUFF
dbCon = None
if dbtype == 'MYSQL':
	try:
		import MySQLdb
	except:
		nagios.tell('UNKNOWN','No postgres module for python found')
	dbCon = MySQLdb.connect(host=server,user=user,passwd=passwd,db=database)
elif dbtype == 'PGSQL':
	# CAN HANDLE 2 DIFFRENT MODULES
	try:
		import pgdb
		conMode = 1
	except:
		try:
			from pyPgSQL import PgSQL
			conMode = 0
		except:
			nagios.tell('UNKNOWN','No postgres module for python found')
	try:
		if conMode == 0:
			dbCon = PgSQL.connect(host=server,database=database,user=user,password=passwd)
		else:
			dbCon = pgdb.connect('%s:%s:%s:%s' % (server,database,user,passwd))
	except:
		nagios.tell('UNKNOWN','No connection possible')
else:
	nagios.tell('UNKNOWN','Databasetype %s not supported' % (dbtype))


if dbCon is not None:
	# READ THE SQLCOMMNADS FROM FILES
	commands = []
	for file in args:
		try:
			f = open(file,'r')
		except:
			nagios.tell('UNKNOWN','Can\'t open file %s' % (file))
		commands += f.read().split('\n')
		f.close()

	# DOING THE SQLCOMMANDS
	cursor = dbCon.cursor()
	start = time()
	for command in commands:
		if command.strip() != '':
			try:
				cursor.execute(command)
			except:
				nagios.tell('UNKNOWN','Can\'t execute query: %s' % (command))

	# PROCESSING THE RESULT
	diffTime = time() - start
	if diffTime < warning:
		nagios.tell('OK','Executed in %0.2f sec (W > %0.2f,C > %0.2f)' % (diffTime,warning,critical))
	elif diffTime < critical:
		nagios.tell('WARNING','Executed in %0.2f sec (W > %0.2f,C > %0.2f)' % (diffTime,warning,critical))
	else:
		nagios.tell('CRITICAL','Executed in %0.2f sec (W > %0.2f,C > %0.2f)' % (diffTime,warning,critical))

else:
	nagios.tell('UNKNOWN','Connection to database could not be created')
